#!/usr/bin/perl

use strict;
use warnings;
use Getopt::Std;

$Getopt::Std::STANDARD_HELP_VERSION = 1;

sub HELP_MESSAGE
{
    print <<EOM;
usage: 4up-booklet.pl [-r] INPDF OUTPDF
       -r .. swap page on left side and right side
EOM
}

sub VERSION_MESSAGE {
    print "4up-booklet.pl version 0.1\n";
}

my %opts = ();
getopts('r', \%opts);

if (@ARGV != 2) {
    &VERSION_MESSAGE();
    &HELP_MESSAGE();
    exit;
}

my $infile = shift(@ARGV);
my $outfile = shift(@ARGV);

my $pages = &get_pdf_pages($infile);

if ($pages <= 0) {
    printf "error!!\n";
    exit;
}

my @map = &book_map($pages);

print join(" ", @map)."\n";

# pdftk A=$infile B=white.pdf cat 1 2 3 4 5 output $outfile

my $cmd = "pdftk A=$infile B=white.pdf cat ";

foreach (@map) {
    if ($_ >= 0) {
	$cmd .= "A".($_+1)." ";
    } else {
	$cmd .= "B1 ";
    }
}
$cmd .= "output tmp-$outfile";

print "EXEC: $cmd\n";
system("$cmd") && exit;

$cmd = "pdfjam tmp-$outfile --nup 2x2 --a3paper --outfile $outfile";
print "EXEC: $cmd\n";
system("$cmd") && exit;

unlink("tmp-$outfile");

sub book_map
{
    my ($pages) = @_;
    my @result;

    my $m = int(($pages + 7) / 8) * 2;

    print "pages: $pages  m: $m\n";

    for (my $p = 0; $p < $m; $p++) {
	my @map;
	@map = ($p       , $m * 4 - $p - 1,
		$m + $p  , $m * 3 - $p - 1);
	if ($map[1] >= $pages) {
	    $map[1] = -1;
	}

	my $reverse = ($p % 2 == 0);
	if ($opts{r}) {
	    $reverse = !$reverse;
	}
	if ($reverse) {
	    ($map[0], $map[1]) = ($map[1], $map[0]);
	    ($map[2], $map[3]) = ($map[3], $map[2]);
	}

	push(@result, @map);

#	print "page: $p\n";
#	printf("+----+----+\n");
#	printf("|%4d|%4d|\n", $map[0], $map[1]);
#	printf("+----+----+\n");
#	printf("|%4d|%4d|\n", $map[2], $map[3]);
#	printf("+----+----+\n");
#	print "\n";
    }
    return @result;
}

sub get_pdf_pages
{
    my ($file) = @_;
    my $pages = -1;

    open(IN, "pdfinfo $file |") || die;
    while (<IN>) {
	if (/Pages:.*?(\d+)/) {
	    $pages = $1 + 0;
	    last;
	}
    }
    close(IN);
    return $pages;
}
